@@ -30,10 +30,11 @@
   */
             
 
-#include "brw_context.h"
 #include "program.h"
-#include "program_instruction.h"
 #include "macros.h"
+#include "shader/prog_parameter.h"
+#include "shader/prog_print.h"
+#include "brw_context.h"
 #include "brw_vs.h"
 
 
@@ -77,8 +78,8 @@ static void brw_vs_alloc_regs( struct brw_vs_compile *c )
    /* Allocate input regs:  
     */
    c->nr_inputs = 0;
-   for (i = 0; i < BRW_ATTRIB_MAX; i++) {
-      if (c->prog_data.inputs_read & ((GLuint64EXT)1<<i)) {
+   for (i = 0; i < VERT_ATTRIB_MAX; i++) {
+      if (c->prog_data.inputs_read & (1<<i)) {
 	 c->nr_inputs++;
 	 c->regs[PROGRAM_INPUT][i] = brw_vec8_grf(reg, 0);
 	 reg++;
@@ -791,7 +792,7 @@ static void emit_vertex_write( struct brw_vs_compile *c)
    if (c->key.copy_edgeflag) {
       brw_MOV(p, 
 	      get_reg(c, PROGRAM_OUTPUT, VERT_RESULT_EDGE),
-	      get_reg(c, PROGRAM_INPUT, BRW_ATTRIB_EDGEFLAG));
+	      get_reg(c, PROGRAM_INPUT, VERT_ATTRIB_EDGEFLAG));
    }
 
 
@@ -844,7 +845,7 @@ static void emit_vertex_write( struct brw_vs_compile *c)
        * Later, clipping will detect ucp[6] and ensure the primitive is
        * clipped against all fixed planes.
        */
-      if (!c->key.know_w_is_one) {
+      if (!BRW_IS_IGD(p->brw) && !c->key.know_w_is_one) {
 	 brw_CMP(p,
 		 vec8(brw_null_reg()),
 		 BRW_CONDITIONAL_L,
