@@ -854,7 +854,8 @@ _mesa_BindTexture( GLenum target, GLuint texName )
             newTexObj = ctx->Shared->DefaultRect;
             break;
          default:
-            ; /* Bad targets are caught above */
+            _mesa_error(ctx, GL_INVALID_ENUM, "glBindTexture(target)");
+            return;
       }
    }
    else {
@@ -926,6 +927,7 @@ _mesa_BindTexture( GLenum target, GLuint texName )
          _mesa_reference_texobj(&texUnit->CurrentRect, newTexObj);
          break;
       default:
+         /* Bad target should be caught above */
          _mesa_problem(ctx, "bad target in BindTexture");
          return;
    }
